<?php

declare(strict_types = 1);

namespace Drupal\Tests\helper\Kernel;

use Drupal\KernelTests\KernelTestBase;

/**
 * Tests the theme helper.
 *
 * @coversDefaultClass \Drupal\helper\Theme
 * @group helper
 */
class ThemeTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'helper',
  ];

  /**
   * The theme helper.
   *
   * @var \Drupal\helper\Theme
   */
  protected $themeHelper;

  /**
   * The available themes.
   *
   * @var string[]
   */
  protected array $themes;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->themeHelper = $this->container->get('helper.theme');
    $this->themes = array_keys($this->container->get('extension.list.theme')->getList());
  }

  /**
   * Tests active theme detection.
   *
   * @covers ::isActive
   * @covers ::setDefault
   *
   * @dataProvider providerIsActive
   */
  public function testIsActive(string $default_theme, array $expected_subthemes = []): void {
    $this->assertNull($this->container->get('theme_handler')->getDefault());
    $this->assertSame('core', $this->container->get('theme.manager')->getActiveTheme()->getName());

    // Test that setDefaultTheme() worked as expected.
    $this->assertTrue($this->themeHelper->setDefault($default_theme));
    $this->assertSame($default_theme, $this->container->get('theme_handler')->getDefault());
    $this->assertSame($default_theme, $this->container->get('theme.manager')->getActiveTheme()->getName());

    // Test that calling setDefaultTheme() again should return false.
    $this->assertFalse($this->themeHelper->setDefault($default_theme));

    // Ensure the expected subthemes are all valid.
    if (!empty($expected_subthemes)) {
      $this->assertEmpty(array_diff($expected_subthemes, $this->themes));
    }

    foreach ($this->themes as $theme_name) {
      if ($theme_name === $default_theme || in_array($theme_name, $expected_subthemes, TRUE)) {
        $this->assertTrue($this->themeHelper->isActive($theme_name), "Asserting that isActive($theme_name) when default theme is $default_theme is TRUE");
      }
      else {
        $this->assertFalse($this->themeHelper->isActive($theme_name), "Asserting that isActive($theme_name) when default theme is $default_theme is FALSE");
      }
    }
  }

  /**
   * Provides test data for ::testIsActive().
   */
  public function providerIsActive(): array {
    $data = [];

    $data[] = [
      'stark',
      [],
    ];

    $data[] = [
      'starterkit_theme',
      [
        'stable9',
      ],
    ];

    $data[] = [
      'test_theme',
      [
        'starterkit_theme',
        'stable9',
      ],
    ];

    $data[] = [
      'test_subsubtheme',
      [
        'test_basetheme',
        'test_subtheme',
      ],
    ];

    return $data;
  }

}
